% fvccomp.m
% Numerical simulation for competition model
% Centered-advection scheme

global duration front advec diffus habitat;
global size1 birth1 adeath1 ldeath1 settle1;
global size2 birth2 adeath2 ldeath2 settle2;
global length time area;
global delx delt xmax tmax;
global xf u k A a1 m1 mu1 nu1 c1 q1 a2 m2 mu2 nu2 c2 q2;
global A1 B1 C1 D1 Beta1 Eps1 Delta1;
global A2 B2 C2 D2 Beta2 Eps2 Delta2;

% Declare biological and numerical variables
fvvars;

% Initialize matrix coefficient vectors
fvcvecs;

L1 = zeros(xmax,tmax);
N1 = zeros(tmax);
L2 = zeros(xmax,tmax);
N2 = zeros(tmax);

% Initialize scaling approximation

% Larval density as a function of distance offshore (y)
% for scaling approximation
 
L1hat = ['(2*n1*m1*(q13*exp(q11*y/(2*k)) - q14*exp(q12*y/(2*k))))/'...
         '((2*c1*(A-a1*n1)+q12)*q13 - (2*c1*(A-a1*n1) + q11)*q14)'];
L2hat = ['(2*n2*m2*(q23*exp(q21*y/(2*k)) - q24*exp(q22*y/(2*k))))/'...
         '((2*c2*(A-a1*n1-a2*n2)+q22)*q23 - (2*c2*(A-a1*n1-a2*n2)'...
         '+ q21)*q24)'];

% Larval density at the coast in the scaling approximation

L1hat0 = '(2*n1*m1*q1)/(2*c1*q1*(A - a1*n1)+ 1)';
L2hat0 = '(2*n2*m2*q2)/(2*c2*q2*(A - a1*n1 - a2*n2)+ 1)';

% Adult equilibria are

p1 = '(2*A*c1*q1*(m1-mu1) - mu1)';
n1hat = 'p1/(2*a1*c1*q1*(m1-mu1))';
n1hat = subs(n1hat,p1,'p1');
n2hat = ['(c1*q1*(m1-mu1)*(a2*p1+a1*mu2) + c2*q2*mu1*(a2*p1 -'...
         'a1*(m2-mu2)))/(2*a2*c1*q1*c2*q2*(m1-mu1)*(a2*p1 -'...
         'a1*(m2-mu2)))'];
n2hat = subs(n2hat,p1,'p1');

% Find expression for equilibrium larval distribution with
% adults at equilibrium:

l1hat = subs(L1hat,n1hat,'n1');
l1hat0 = subs(L1hat0,n1hat,'n1');
l1hat0 = simple(l1hat0);
l2hat = subs(subs(L2hat,n2hat,'n2'),n1hat,'n1');
l2hat0 = subs(subs(L2hat0,n2hat,'n2'),n1hat,'n1');
l2hat0 = simple(l2hat0);

% Equilibrium larval population given initial adult population

L1init = subs(L1hat,'start1','n1');
L2init = subs(subs(L2hat,'start2','n2'),'start1','n1');

L1hat = sym2ara(L1hat);
L1init = sym2ara(L1init);
l1hat = sym2ara(l1hat);
L2hat = sym2ara(L2hat);
L2init = sym2ara(L2init);
l2hat = sym2ara(l2hat);

% Spatial counting variable for volumes:

x = 1:xmax;

% Conversion formula from spatial step to space coordinate
% appropriate to analytical formula (scaling approx.):

y = delx*(x-1/2);

% obtain the q's:

q11 = u + sqrt(4*k*nu1 + u^2);
q12 = u - sqrt(4*k*nu1 + u^2);
q13 = q11*exp(q12*xf/(2*k));
q14 = q12*exp(q11*xf/(2*k));
q1 = (q13-q14)/(q12*q13 - q11*q14);
q21 = u + sqrt(4*k*nu2 + u^2);
q22 = u - sqrt(4*k*nu2 + u^2);
q23 = q21*exp(q22*xf/(2*k));
q24 = q22*exp(q21*xf/(2*k));
q2 = (q23-q24)/(q22*q23 - q21*q24);

% Evaluate equilibria (abundances and percent cover):

eq1 = eval(n1hat)
eqpc1 = 100*size1*eq1/habitat
eq2 = eval(n2hat)
eqpc2 = 100*size2*eq2/habitat

% Initial conditions for simulations (as percent cover):
% Note: these are the initial conditions used to plot
% the trajectories in Fig. 1.

start11 = 10;
start12 = 80;
start21 = 10;
start22 = 80;

% Initial conditions for this simulation, converted to
% abundances.  Note that these values will plot trajectories
% from only one set of initial conditions (here, starting
% at 10 percent cover of both species).  These values will
% have to be changed (e.g., substituting start12 for start11
% to start at 80 percent dominant cover and 10 percent
% subordinate cover) to plot the other trajectories.

start1 = start11.*A./a1./100;
start2 = start21.*A./a2./100;

% Initialize adult populations for simulations
% Ni is full model, SNi is scaling approximation:

N1(1) = start1;
SN1(1) = N1(1);
N2(1) = start2;
SN2(1) = N2(1);

% Initialize larval abundances for full model (so that
% it starts in the same state as the scaling approximation):

l1init = eval(L1init);
L1(:,1) = l1init';
l2init = eval(L2init);
L2(:,1) = l2init';

Eps1(1) = 0;
Eps2(1) = 0;

for t=2:tmax
   
   % Step scaling approximation

   n1 = SN1(t-1);
   n2 = SN2(t-1);
   L10 = eval(L1hat0); % larval equilibria based on adult 
   L20 = eval(L2hat0); % abundances at previous time step
   SN1(t) = (SN1(t-1)/delt + c1*A*L10)/(1/delt + c1*a1*L10 + mu1);
   SN2(t) = (SN2(t-1)/delt + c2*(A-a1*SN1(t))*L20)/...
            (1/delt + c2*a2*L20 + mu2 + c1*L10*a2);

   % Step adult dynamics of full model:
   N1(t) = (N1(t-1)/delt + c1*A*L1(1,t-1))/(1/delt + ...
           c1*a1*L1(1,t-1) + mu1);
   N2(t) = (N2(t-1)/delt + c2*(A-a1*N1(t))*L2(1,t-1))/...
           (1/delt + c2*a2*L2(1,t-1) + mu2 + c1*L1(1,t-1)*a2);

   % obtain N1-dependent matrix elements for first row:
   B1(1) = 1/delt + u/(2*delx) + k/delx^2 + c1*(A - a1*N1(t))/...
           delx + nu1;
   B2(1) = 1/delt + u/(2*delx) + k/delx^2 + c2*(A - a1*N1(t) -...
           a2*N2(t))/delx + nu2;
   D1(1) = m1*N1(t)/delx + L1(1,t-1)/delt;
   D2(1) = m2*N2(t)/delx + L2(1,t-1)/delt;
   Beta1(1) = B1(1);
   Beta2(1) = B2(1);
   Delta1(1) = D1(1);
   Delta2(1) = D2(1);
   
   % Calculate new matrix with elements beta, c, delta:
   for x=2:xmax
     D1(x) = L1(x,t-1)/delt;
     D2(x) = L2(x,t-1)/delt;
     Eps1(x) = A1(x)/Beta1(x-1);
     Eps2(x) = A2(x)/Beta2(x-1);
     Beta1(x) = B1(x) - Eps1(x)*C1(x-1);
     Beta2(x) = B2(x) - Eps2(x)*C2(x-1);
     Delta1(x) = D1(x) - Eps1(x)*Delta1(x-1);
     Delta2(x) = D2(x) - Eps2(x)*Delta2(x-1);
   end
   
   % Backsubstitute to obtain larval concentrations:
   L1(xmax,t) = Delta1(xmax)/Beta1(xmax);
   L2(xmax,t) = Delta2(xmax)/Beta2(xmax);
   for x=xmax-1:-1:1
     L1(x,t) = (Delta1(x) - C1(x)*L1(x+1,t))/Beta1(x);
     L2(x,t) = (Delta2(x) - C2(x)*L2(x+1,t))/Beta2(x);
   end
   
end

figure 
hold on
set(gca,'Xlim',[0 100],'Ylim',[0 100],'FontSize',18,'LineWidth',1)
xlabel('Dominant Percent Cover','FontSize',22)
ylabel('Subordinate Percent Cover','FontSize',22)
title('Full Model vs. Scaling Approximation','FontSize',22)
plot(eqpc1,eqpc2,'*') % put asterisks at equilibria

% Note: to plot multiple trajectories on the same figure, as in
% Fig. 1, comment out the above lines ('figure' statement down
% to this comment) and change the initial population sizes after 
% plotting the first set of trajectories, then re-run the script
% with the original figure window still active (see MATLAB
% documentation on plotting multiple graphics in a single figure
% window).

t=1:tmax;
hold on
plot(100.*a1.*N1(t)./A,100.*a2.*N2(t)./A,'LineWidth',1.5)
plot(100.*a1.*SN1(t)./A,100.*a2.*SN2(t)./A,'--','LineWidth',1.5)