% fvcpp.m
% Numerical simulation for predator-prey model
% Centered-advection scheme

global duration front advec diffus habitat encount;
global size1 birth1 adeath1 ldeath1 settle1;
global birth2 adeath2 ldeath2 settle2;
global length time area;
global delx delt xmax tmax;
global xf u k A e a1 m1 mu1 nu1 c1 q1 m2 mu2 nu2 c2 q2;
global A1 B1 C1 D1 Beta1 Eps1 Delta1;
global A2 B2 C2 D2 Beta2 Eps2 Delta2;
global N1o L1o N2o L2o;

% Declare biological and numerical variables
fvvarspp;

% Initialize matrix coefficient vectors
fvcvecspp;

L1 = zeros(xmax,tmax);
N1 = zeros(tmax);
L2 = zeros(xmax,tmax);
N2 = zeros(tmax);

% Initialize scaling approximation

% Larval density as a function of distance offshore (y)
% for the scaling approximation

L1hat = ['(2*n1*m1*(q13*exp(q11*y/(2*k)) - q14*exp(q12*y/(2*k))))/'...
         '((2*c1*(A-a1*n1)+q12)*q13 - (2*c1*(A-a1*n1) + q11)*q14)'];
L2hat = ['(2*n1*n2*m2*e*(q23*exp(q21*y/(2*k)) -'...
          'q24*exp(q22*y/(2*k))))/'...
         '((2*c2*A+q22)*q23 - (2*c2*A + q21)*q24)'];

% Larval density at the coast in the scaling approximation
 
L1hat0 = '(2*n1*m1*q1)/(2*c1*q1*(A - a1*n1)+ 1)';
L2hat0 = '(2*n1*n2*m2*e*q2)/(2*c2*q2*A + 1)';

% Adult equilibria are

n1hat = 'mu2*(2*c2*A*q2 + 1)/(2*A*c2*q2*e*m2)';
n2hat = ['(c1*q1*(m1-mu1)*(2*c2*A*q2*(A*e*m2-a1*mu2) - a1*mu2)'...
   '- c2*A*q2*e*m2*mu1)/'...
   '(e*(c1*q1*(2*A*c2*q2*(A*e*m2-a1*mu2) - a1*mu2) + A*c2*q2*e*m2))'];

% Find expression for equilibrium larval distribution with
% adults at equilibrium:

l1hat = subs(L1hat,n1hat,'n1');
l1hat0 = subs(L1hat0,n1hat,'n1');
l1hat0 = simple(l1hat0);
l2hat = subs(subs(L2hat,n2hat,'n2'),n1hat,'n1');
l2hat0 = subs(subs(L2hat0,n2hat,'n2'),n1hat,'n1');
l2hat0 = simple(l2hat0);

% Equilibrium larval population given initial adult population

L1init = subs(L1hat,'start1','n1');
L2init = subs(subs(L2hat,'start2','n2'),'start1','n1');

L1hat = sym2ara(L1hat);
L1init = sym2ara(L1init);
l1hat = sym2ara(l1hat);
L2hat = sym2ara(L2hat);
L2init = sym2ara(L2init);
l2hat = sym2ara(l2hat);

% Spatial counting variable for volumes

x = 1:xmax;

% Conversion formula from spatial step to space coordinate
% appropriate to analytical formula (scaling approx.):

y = delx*(x-1/2);

% obtain the q's:

q11 = u + sqrt(4*k*nu1 + u^2);
q12 = u - sqrt(4*k*nu1 + u^2);
q13 = q11*exp(q12*xf/(2*k));
q14 = q12*exp(q11*xf/(2*k));
q1 = (q13-q14)/(q12*q13 - q11*q14);
q21 = u + sqrt(4*k*nu2 + u^2);
q22 = u - sqrt(4*k*nu2 + u^2);
q23 = q21*exp(q22*xf/(2*k));
q24 = q22*exp(q21*xf/(2*k));
q2 = (q23-q24)/(q22*q23 - q21*q24);

% Evaluate equilibria (abundances, percent cover,
% and density):

eq1 = eval(n1hat)
eqpc1 = 100*size1*eq1/habitat
eq2 = eval(n2hat)
eqd2 = eq2/habitat

% Initial conditions for simulations (as percent cover):
% Note: these are the initial conditions used to plot
% the trajectories in Fig. 6.

start11 = eq1/2;
start12 = eq1*2;
start21 = eq2/2;
start22 = eq2*2;

% Initial conditions for this simulation, converted to
% abundances.  Note that these values will plot trajectories
% from only one set of initial conditions (here, starting
% at half the equilibrium abundances for both species).  These
% values will have to be changed (e.g., substituting start12
% for start11 to start the prey at twice its equilibrium
% abundance) to plot the other trajectories in Fig. 6.

start1 = start12;
start2 = start22;

% Initialize adult populations for simulations
% Ni is full model, SNi is scaling approximation:

N1(1) = start1;
SN1(1) = N1(1);
N2(1) = start2;
SN2(1) = N2(1);

% Initialize larval abundances for full model (so that
% it starts in the same state as the scaling approximation):

l1init = eval(L1init);
L1(:,1) = l1init';
l2init = eval(L2init);
L2(:,1) = l2init';


Eps1(1) = 0;
Eps2(1) = 0;

for t=2:tmax
   
   % Step scaling approximation

   n1 = SN1(t-1);
   n2 = SN2(t-1);
   L10 = eval(L1hat0);
   L20 = eval(L2hat0);
   SN2(t) = (SN2(t-1)/delt + c2*A*L20)/(1/delt + mu2);
   SN1(t) = (SN1(t-1)/delt + c1*A*L10)/(1/delt + c1*a1*L10 +...
            mu1 + e*SN2(t));

   % Step adult dynamics of full model:
   N2(t) = (N2(t-1)/delt + c2*A*L2(1,t-1))/(1/delt + mu2);
   N1(t) = (N1(t-1)/delt + c1*A*L1(1,t-1))/(1/delt + ...
            c1*a1*L1(1,t-1) + mu1 + e*N2(t));

   % Obtain N1-dependent matrix elements for first row:
   B1(1) = 1/delt + u/(2*delx) + k/delx^2 + c1*(A -...
           a1*N1(t))/delx + nu1;
   B2(1) = 1/delt + u/(2*delx) + k/delx^2 + c2*A/delx + nu2;
   D1(1) = m1*N1(t)/delx + L1(1,t-1)/delt;
   D2(1) = m2*e*N1(t)*N2(t)/delx + L2(1,t-1)/delt;
   Beta1(1) = B1(1);
   Beta2(1) = B2(1);
   Delta1(1) = D1(1);
   Delta2(1) = D2(1);
   
   % Calculate new matrix with elements beta, c, delta:
   for x=2:xmax
     D1(x) = L1(x,t-1)/delt;
     D2(x) = L2(x,t-1)/delt;
     Eps1(x) = A1(x)/Beta1(x-1);
     Eps2(x) = A2(x)/Beta2(x-1);
     Beta1(x) = B1(x) - Eps1(x)*C1(x-1);
     Beta2(x) = B2(x) - Eps2(x)*C2(x-1);
     Delta1(x) = D1(x) - Eps1(x)*Delta1(x-1);
     Delta2(x) = D2(x) - Eps2(x)*Delta2(x-1);
   end
   
   % Backsubstitute to obtain larval concentrations:
   L1(xmax,t) = Delta1(xmax)/Beta1(xmax);
   L2(xmax,t) = Delta2(xmax)/Beta2(xmax);
   for x=xmax-1:-1:1
     L1(x,t) = (Delta1(x) - C1(x)*L1(x+1,t))/Beta1(x);
     L2(x,t) = (Delta2(x) - C2(x)*L2(x+1,t))/Beta2(x);
   end
   
end

figure
hold on
set(gca,'Xlim',[0 40],'Ylim',[0 0.8],'FontSize',18,'LineWidth',1)
xlabel('Prey Percent Cover','FontSize',22)
title('Full Model vs. Scaling Approximation','FontSize',22)
plot(eqpc1,eqd2,'*')

% To plot multiple trajectories on the same figure with this script,
% see the comment in the corresponding location in fvccomp.m

t=1:round(tmax/2);
plot(100.*size1.*N1(2*t)./habitat,N2(2*t)./habitat,'LineWidth',1.5)
plot(100.*size1.*SN1(2*t)./habitat,SN2(2*t)./habitat,'--',...
     'LineWidth',1.5)
